# Copyright 2020 Nokia Software.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from unittest import mock


from mistralclient.api.v2 import dynamic_actions
from mistralclient.commands.v2 import dynamic_actions as dyn_actions_cmd
from mistralclient.tests.unit import base


DYN_ACTION_DICT = {
    'id': '1-2-3-4',
    'name': 'my_action',
    'class_name': 'MyAction',
    'code_source_name': 'my_module',
    'code_source_id': '2-3-4-5',
    'namespace': '',
    'project_id': '12345',
    'scope': 'private',
    'created_at': '1',
    'updated_at': '1'
}

DYN_ACTION = dynamic_actions.DynamicAction(mock, DYN_ACTION_DICT)


class TestCLIDynamicActions(base.BaseCommandTest):
    @mock.patch('argparse.open', create=True)
    def test_create(self, mock_open):
        self.client.dynamic_actions.create.return_value = DYN_ACTION

        result = self.call(
            dyn_actions_cmd.Create,
            app_args=['my_action', 'MyAction', 'my_module']
        )

        self.assertEqual(
            (
                '1-2-3-4',
                'my_action',
                'MyAction',
                '2-3-4-5',
                'my_module',
                '12345',
                'private',
                '1',
                '1'
            ),
            result[1]
        )

    @mock.patch('argparse.open', create=True)
    def test_create_public(self, mock_open):
        dyn_action_dict = DYN_ACTION_DICT.copy()
        dyn_action_dict['scope'] = 'public'

        dyn_action = dynamic_actions.DynamicAction(mock, dyn_action_dict)

        self.client.dynamic_actions.create.return_value = dyn_action

        result = self.call(
            dyn_actions_cmd.Create,
            app_args=['my_action', 'MyAction', 'my_module', '--public']
        )

        self.assertEqual(
            (
                '1-2-3-4',
                'my_action',
                'MyAction',
                '2-3-4-5',
                'my_module',
                '12345',
                'public',
                '1',
                '1'
            ),
            result[1]
        )
        self.assertEqual(
            'public',
            self.client.dynamic_actions.create.call_args[1]['scope']
        )

    @mock.patch('argparse.open', create=True)
    def test_update(self, mock_open):
        dyn_action_dict = DYN_ACTION_DICT.copy()
        dyn_action_dict['class_name'] = 'MyNewAction'

        dyn_action = dynamic_actions.DynamicAction(mock, dyn_action_dict)

        self.client.dynamic_actions.update.return_value = dyn_action

        result = self.call(
            dyn_actions_cmd.Update,
            app_args=['my_action', '--class-name', 'MyNewAction']
        )

        self.assertEqual(
            (
                '1-2-3-4',
                'my_action',
                'MyNewAction',
                '2-3-4-5',
                'my_module',
                '12345',
                'private',
                '1',
                '1'
            ),
            result[1]
        )

    @mock.patch('argparse.open', create=True)
    def test_update_with_id(self, mock_open):
        dyn_action_dict = DYN_ACTION_DICT.copy()
        dyn_action_dict['class_name'] = 'MyNewAction'

        dyn_action = dynamic_actions.DynamicAction(mock, dyn_action_dict)

        self.client.dynamic_actions.update.return_value = dyn_action

        result = self.call(
            dyn_actions_cmd.Update,
            app_args=['1-2-3-4', '--class-name', 'MyNewAction']
        )

        self.assertEqual(
            (
                '1-2-3-4',
                'my_action',
                'MyNewAction',
                '2-3-4-5',
                'my_module',
                '12345',
                'private',
                '1',
                '1'
            ),
            result[1]
        )

    def test_list(self):
        self.client.dynamic_actions.list.return_value = [DYN_ACTION]

        result = self.call(dyn_actions_cmd.List)

        self.assertEqual(
            [
                (
                    '1-2-3-4',
                    'my_action',
                    'MyAction',
                    '2-3-4-5',
                    'my_module',
                    '12345',
                    'private',
                    '1',
                    '1'
                )
            ],
            result[1]
        )

    def test_get(self):
        self.client.dynamic_actions.get.return_value = DYN_ACTION

        result = self.call(dyn_actions_cmd.Get, app_args=['name'])

        self.assertEqual(
            (
                '1-2-3-4',
                'my_action',
                'MyAction',
                '2-3-4-5',
                'my_module',
                '12345',
                'private',
                '1',
                '1'
            ),
            result[1]
        )

    def test_delete(self):
        self.call(dyn_actions_cmd.Delete, app_args=['my_action'])

        self.client.dynamic_actions.delete.assert_called_once_with(
            'my_action',
            namespace=''
        )

    def test_delete_with_multi_names(self):
        self.call(dyn_actions_cmd.Delete, app_args=['name1', 'name2'])

        self.assertEqual(2, self.client.dynamic_actions.delete.call_count)
        self.assertEqual(
            [
                mock.call('name1', namespace=''),
                mock.call('name2', namespace='')
            ],
            self.client.dynamic_actions.delete.call_args_list
        )



# Copyright 2014 - Mirantis, Inc.
# Copyright 2015 - StackStorm, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from mistralclient.api import base


class Workbook(base.Resource):
    resource_name = 'Workbook'


class WorkbookManager(base.ResourceManager):
    resource_class = Workbook

    def _get_workbooks_url(self, resource=None, namespace=None, scope=None):
        url = '/workbooks'

        if resource:
            url += '/%s' % resource

        if scope and namespace:
            url += '?scope=%s&namespace=%s' % (scope, namespace)
        elif scope:
            url += '?scope=%s' % scope
        elif namespace:
            url += '?namespace=%s' % namespace

        return url

    def create(self, definition, namespace='', scope='private'):
        self._ensure_not_empty(definition=definition)

        # If the specified definition is actually a file, read in the
        # definition file
        definition = self.get_contents_if_file(definition)

        return self._create(
            self._get_workbooks_url(None, namespace, scope),
            definition,
            dump_json=False,
            headers={'content-type': 'text/plain'}
        )

    def update(self, definition, namespace='', scope='private'):
        self._ensure_not_empty(definition=definition)

        # If the specified definition is actually a file, read in the
        # definition file
        definition = self.get_contents_if_file(definition)

        return self._update(
            self._get_workbooks_url(None, namespace, scope),
            definition,
            dump_json=False,
            headers={'content-type': 'text/plain'}
        )

    def list(self, namespace='', marker='', limit=None, sort_keys='',
             sort_dirs='', fields='', **filters):
        query_string = self._build_query_params(
            marker=marker,
            limit=limit,
            sort_keys=sort_keys,
            sort_dirs=sort_dirs,
            filters=filters,
            namespace=namespace
        )

        return self._list(
            '/workbooks{}'.format(query_string),
            response_key='workbooks'
        )

    def get(self, name, namespace=''):
        self._ensure_not_empty(name=name)

        return self._get(
            self._get_workbooks_url(name, namespace)
        )

    def delete(self, name, namespace=''):
        self._ensure_not_empty(name=name)

        self._delete(self._get_workbooks_url(name, namespace))

    def validate(self, definition):
        self._ensure_not_empty(definition=definition)

        # If the specified definition is actually a file, read in the
        # definition file
        definition = self.get_contents_if_file(definition)

        return self._validate(
            '/workbooks/validate',
            definition,
            dump_json=False,
            headers={'content-type': 'text/plain'}
        )



# Copyright 2017, OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from osc_lib.command import command

from mistralclient.commands.v2 import base
from mistralclient import utils


class EventTriggerFormatter(base.MistralFormatter):
    COLUMNS = [
        ('id', 'ID'),
        ('name', 'Name'),
        ('workflow_id', 'Workflow ID'),
        ('workflow_params', 'Params'),
        ('exchange', 'Exchange'),
        ('topic', 'Topic'),
        ('event', 'Event'),
        ('created_at', 'Created at'),
        ('updated_at', 'Updated at')
    ]

    @staticmethod
    def format(trigger=None, lister=False):
        if trigger:
            data = (
                trigger.id,
                trigger.name,
                trigger.workflow_id,
                trigger.workflow_params,
                trigger.exchange,
                trigger.topic,
                trigger.event,
                trigger.created_at,
            )

            if hasattr(trigger, 'updated_at'):
                data += (trigger.updated_at,)
            else:
                data += (None,)
        else:
            data = (tuple('' for _ in
                          range(len(EventTriggerFormatter.COLUMNS))),)

        return EventTriggerFormatter.headings(), data


class List(base.MistralLister):
    """List all event triggers."""

    def _get_format_function(self):
        return EventTriggerFormatter.format_list

    def _get_resources(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine
        return mistral_client.event_triggers.list(
            marker=parsed_args.marker,
            limit=parsed_args.limit,
            sort_keys=parsed_args.sort_keys,
            sort_dirs=parsed_args.sort_dirs,
            fields=EventTriggerFormatter.fields(),
            **base.get_filters(parsed_args)
        )


class Get(command.ShowOne):
    """Show specific event trigger."""

    def get_parser(self, prog_name):
        parser = super(Get, self).get_parser(prog_name)

        parser.add_argument('event_trigger', help='Event trigger ID')

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine

        return EventTriggerFormatter.format(mistral_client.event_triggers.get(
                                            parsed_args.event_trigger))


class Create(command.ShowOne):
    """Create new trigger."""

    def get_parser(self, prog_name):
        parser = super(Create, self).get_parser(prog_name)

        parser.add_argument('name', help='Event trigger name')
        parser.add_argument('workflow_id', help='Workflow ID')

        parser.add_argument('exchange',
                            type=str,
                            help='Event trigger exchange')

        parser.add_argument('topic',
                            type=str,
                            help='Event trigger topic')

        parser.add_argument('event',
                            type=str,
                            help='Event trigger event name')

        parser.add_argument('workflow_input',
                            nargs='?',
                            help='Workflow input')

        parser.add_argument('--params',
                            help='Workflow params')

        return parser

    @staticmethod
    def _get_json_string_or_dict(string):
        if string:
            return utils.load_json(string)
        else:
            return {}

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine

        wf_input = self._get_json_string_or_dict(parsed_args.workflow_input)
        wf_params = self._get_json_string_or_dict(parsed_args.params)

        trigger = mistral_client.event_triggers.create(
            parsed_args.name,
            parsed_args.workflow_id,
            parsed_args.exchange,
            parsed_args.topic,
            parsed_args.event,
            wf_input,
            wf_params,
        )

        return EventTriggerFormatter.format(trigger)


class Delete(command.Command):
    """Delete trigger."""

    def get_parser(self, prog_name):
        parser = super(Delete, self).get_parser(prog_name)

        parser.add_argument(
            'event_trigger_id',
            nargs='+', help='ID of event trigger(s).'
        )

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine

        utils.do_action_on_many(
            lambda s: mistral_client.event_triggers.delete(s),
            parsed_args.event_trigger_id,
            "Request to delete event trigger %s has been accepted.",
            "Unable to delete the specified event trigger(s)."
        )



# Mistral documentation build configuration file

import os
import sys

on_rtd = os.environ.get('READTHEDOCS', None) == 'True'

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, os.path.abspath('../../'))
sys.path.insert(0, os.path.abspath('../'))
sys.path.insert(0, os.path.abspath('./'))

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    'sphinxcontrib.apidoc',
    'openstackdocstheme',
]

# Add any paths that contain templates here, relative to this directory.
# templates_path = ['_templates']

# sphinxcontrib.apidoc options
apidoc_module_dir = '../../mistralclient'
apidoc_output_dir = 'api'
apidoc_excluded_paths = [
    'test',
    'tests/*']
apidoc_separate_modules = True

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'Mistral Client'
copyright = '2016, Mistral Contributors'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = []

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'native'

# A list of ignored prefixes for module index sorting.
modindex_common_prefix = ['mistralclient.']


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'openstackdocs'

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
html_title = 'MistralClient'

# Custom sidebar templates, maps document names to template names.
html_sidebars = {
    'index': [
        'sidebarlinks.html', 'localtoc.html', 'searchbox.html',
        'sourcelink.html'
    ],
    '**': [
        'localtoc.html', 'relations.html',
        'searchbox.html', 'sourcelink.html'
    ]
}

# Output file base name for HTML help builder.
htmlhelp_basename = 'Mistraldoc'


# -- Options for manual page output ------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('index', 'mistral_client', 'Mistral Client Documentation',
    ['Mistral Contributors'], 1)
]

# -- Options for openstackdocstheme -------------------------------------------
openstackdocs_repo_name = 'openstack/python-mistralclient'
openstackdocs_bug_project = 'python-mistralclient'
openstackdocs_bug_tag = ''
openstackdocs_auto_name = False



# Copyright 2015 Huawei Technologies Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import os.path
from urllib import parse
from urllib import request

from mistralclient.api.v2 import workflows
from mistralclient.tests.unit.v2 import base


WF_DEF = """
---
version: 2.0

my_wf:
  type: direct

  tasks:
    task1:
      action: std.echo output="hello, world"
"""

WORKFLOW = {
    'id': '123',
    'name': 'my_wf',
    'input': '',
    'definition': WF_DEF
}

URL_TEMPLATE = '/workflows'
URL_TEMPLATE_SCOPE = '/workflows?scope=private'
URL_TEMPLATE_NAME = '/workflows/%s'


class TestWorkflowsV2(base.BaseClientV2Test):
    def test_create(self):
        self.requests_mock.post(
            self.TEST_URL + URL_TEMPLATE_SCOPE,
            json={'workflows': [WORKFLOW]},
            status_code=201
        )

        wfs = self.workflows.create(WF_DEF)

        self.assertIsNotNone(wfs)
        self.assertEqual(WF_DEF, wfs[0].definition)

        last_request = self.requests_mock.last_request

        self.assertEqual(WF_DEF, last_request.text)
        self.assertEqual('text/plain', last_request.headers['content-type'])

    def test_create_with_file(self):
        self.requests_mock.post(
            self.TEST_URL + URL_TEMPLATE_SCOPE,
            json={'workflows': [WORKFLOW]},
            status_code=201
        )

        # The contents of wf_v2.yaml must be identical to WF_DEF
        path = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                            '..', 'resources', 'wf_v2.yaml')

        wfs = self.workflows.create(path)

        self.assertIsNotNone(wfs)
        self.assertEqual(WF_DEF, wfs[0].definition)

        last_request = self.requests_mock.last_request

        self.assertEqual(WF_DEF, last_request.text)
        self.assertEqual('text/plain', last_request.headers['content-type'])

    def test_update(self):
        self.requests_mock.put(
            self.TEST_URL + URL_TEMPLATE_SCOPE,
            json={'workflows': [WORKFLOW]}
        )

        wfs = self.workflows.update(WF_DEF)

        self.assertIsNotNone(wfs)
        self.assertEqual(WF_DEF, wfs[0].definition)

        last_request = self.requests_mock.last_request

        self.assertEqual(WF_DEF, last_request.text)
        self.assertEqual('text/plain', last_request.headers['content-type'])

    def test_update_with_id(self):
        self.requests_mock.put(
            self.TEST_URL + URL_TEMPLATE_NAME % '123',
            json=WORKFLOW
        )

        wf = self.workflows.update(WF_DEF, id='123')

        self.assertIsNotNone(wf)
        self.assertEqual(WF_DEF, wf.definition)

        last_request = self.requests_mock.last_request

        self.assertEqual('namespace=&scope=private', last_request.query)
        self.assertEqual(WF_DEF, last_request.text)
        self.assertEqual('text/plain', last_request.headers['content-type'])

    def test_update_with_file_uri(self):
        self.requests_mock.put(
            self.TEST_URL + URL_TEMPLATE_SCOPE,
            json={'workflows': [WORKFLOW]}
        )

        # The contents of wf_v2.yaml must be identical to WF_DEF
        path = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                            '..', 'resources', 'wf_v2.yaml')

        path = os.path.abspath(path)

        # Convert the file path to file URI
        uri = parse.urljoin('file:', request.pathname2url(path))

        wfs = self.workflows.update(uri)

        self.assertIsNotNone(wfs)
        self.assertEqual(WF_DEF, wfs[0].definition)

        last_request = self.requests_mock.last_request

        self.assertEqual(WF_DEF, last_request.text)
        self.assertEqual('text/plain', last_request.headers['content-type'])

    def test_list(self):
        self.requests_mock.get(
            self.TEST_URL + URL_TEMPLATE,
            json={'workflows': [WORKFLOW]}
        )

        workflows_list = self.workflows.list()

        self.assertEqual(1, len(workflows_list))

        wf = workflows_list[0]

        self.assertEqual(
            workflows.Workflow(self.workflows, WORKFLOW).to_dict(),
            wf.to_dict()
        )

    def test_list_with_pagination(self):
        self.requests_mock.get(
            self.TEST_URL + URL_TEMPLATE,
            json={
                'workflows': [WORKFLOW],
                'next': '/workflows?fake'
            }
        )

        workflows_list = self.workflows.list(
            limit=1,
            sort_keys='created_at',
            sort_dirs='asc'
        )

        self.assertEqual(1, len(workflows_list))

        last_request = self.requests_mock.last_request

        # The url param order is unpredictable.
        self.assertEqual(['1'], last_request.qs['limit'])
        self.assertEqual(['created_at'], last_request.qs['sort_keys'])
        self.assertEqual(['asc'], last_request.qs['sort_dirs'])

    def test_list_with_no_limit(self):
        self.requests_mock.get(
            self.TEST_URL + URL_TEMPLATE,
            json={'workflows': [WORKFLOW]}
        )

        workflows_list = self.workflows.list(limit=-1)

        self.assertEqual(1, len(workflows_list))

        last_request = self.requests_mock.last_request

        self.assertNotIn('limit', last_request.qs)

    def test_get(self):
        url = self.TEST_URL + URL_TEMPLATE_NAME % 'wf'

        self.requests_mock.get(url, json=WORKFLOW)

        wf = self.workflows.get('wf')

        self.assertIsNotNone(wf)
        self.assertEqual(
            workflows.Workflow(self.workflows, WORKFLOW).to_dict(),
            wf.to_dict()
        )

    def test_delete(self):
        self.requests_mock.delete(
            self.TEST_URL + URL_TEMPLATE_NAME % 'wf',
            status_code=204
        )

        self.workflows.delete('wf')
